@@ -2,6 +2,7 @@
  * dlfcn-win32
  * Copyright (c) 2007 Ramiro Polla
  * Copyright (c) 2015 Tiancheng "Timothy" Gu
+ * Copyright (c) 2019 Pali Rohár <pali.rohar@gmail.com>
  *
  * dlfcn-win32 is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -27,6 +28,18 @@
 #include <windows.h>
 #include <psapi.h>
 #include <stdio.h>
+#include <stdlib.h>
+
+#ifdef _MSC_VER
+/* https://docs.microsoft.com/en-us/cpp/intrinsics/returnaddress */
+#include <intrin.h>
+#pragma intrinsic(_ReturnAddress)
+#else
+/* https://gcc.gnu.org/onlinedocs/gcc/Return-Address.html */
+#ifndef _ReturnAddress
+#define _ReturnAddress() (__builtin_extract_return_addr(__builtin_return_address(0)))
+#endif
+#endif
 
 #ifdef SHARED
 #define DLFCN_WIN32_EXPORTS
@@ -51,57 +64,48 @@
  * any kind of thread safety.
  */
 
-typedef struct global_object {
+typedef struct local_object {
     HMODULE hModule;
-    struct global_object *previous;
-    struct global_object *next;
-} global_object;
+    struct local_object *previous;
+    struct local_object *next;
+} local_object;
 
-static global_object first_object;
-static global_object first_automatic_object;
-static int auto_ref_count = 0;
+static local_object first_object;
 
-/* These functions implement a double linked list for the global objects. */
-static global_object *global_search( global_object *start, HMODULE hModule )
+/* These functions implement a double linked list for the local objects. */
+static local_object *local_search( HMODULE hModule )
 {
-    global_object *pobject;
+    local_object *pobject;
 
     if( hModule == NULL )
         return NULL;
 
-    for( pobject = start; pobject; pobject = pobject->next )
+    for( pobject = &first_object; pobject; pobject = pobject->next )
         if( pobject->hModule == hModule )
             return pobject;
 
     return NULL;
 }
 
-static void global_add( global_object *start, HMODULE hModule )
+static void local_add( HMODULE hModule )
 {
-    global_object *pobject;
-    global_object *nobject;
+    local_object *pobject;
+    local_object *nobject;
 
     if( hModule == NULL )
         return;
 
-    pobject = global_search( start, hModule );
+    pobject = local_search( hModule );
 
     /* Do not add object again if it's already on the list */
     if( pobject )
         return;
 
-    if( start == &first_automatic_object )
-    {
-        pobject = global_search( &first_object, hModule );
-        if( pobject )
-            return;
-    }
+    for( pobject = &first_object; pobject->next; pobject = pobject->next );
 
-    for( pobject = start; pobject->next; pobject = pobject->next );
+    nobject = (local_object*) malloc( sizeof( local_object ) );
 
-    nobject = (global_object*) malloc( sizeof( global_object ) );
-
-    /* Should this be enough to fail global_add, and therefore also fail
+    /* Should this be enough to fail local_add, and therefore also fail
      * dlopen?
      */
     if( !nobject )
@@ -113,14 +117,14 @@ static void global_add( global_object *start, HMODULE hModule )
     nobject->hModule = hModule;
 }
 
-static void global_rem( global_object *start, HMODULE hModule )
+static void local_rem( HMODULE hModule )
 {
-    global_object *pobject;
+    local_object *pobject;
 
     if( hModule == NULL )
         return;
 
-    pobject = global_search( start, hModule );
+    pobject = local_search( hModule );
 
     if( !pobject )
         return;
@@ -223,10 +227,6 @@ void *dlopen( const char *file, int mode )
 
     if( file == 0 )
     {
-        HMODULE hAddtnlMods[1024]; // Already loaded modules
-        HANDLE hCurrentProc = GetCurrentProcess( );
-        DWORD cbNeeded;
-
         /* POSIX says that if the value of file is 0, a handle on a global
          * symbol object must be provided. That object must be able to access
          * all symbols from the original program file, and any objects loaded
@@ -241,27 +241,13 @@ void *dlopen( const char *file, int mode )
 
         if( !hModule )
             save_err_ptr_str( file );
-
-
-        /* GetModuleHandle( NULL ) only returns the current program file. So
-         * if we want to get ALL loaded module including those in linked DLLs,
-         * we have to use EnumProcessModules( ).
-         */
-        if( EnumProcessModules( hCurrentProc, hAddtnlMods,
-            sizeof( hAddtnlMods ), &cbNeeded ) != 0 )
-        {
-            DWORD i;
-            for( i = 0; i < cbNeeded / sizeof( HMODULE ); i++ )
-            {
-                global_add( &first_automatic_object, hAddtnlMods[i] );
-            }
-        }
-        auto_ref_count++;
     }
     else
     {
+        HANDLE hCurrentProc;
+        DWORD dwProcModsBefore, dwProcModsAfter;
         CHAR lpFileName[MAX_PATH];
-        int i;
+        size_t i;
 
         /* MSDN says backslashes *must* be used instead of forward slashes. */
         for( i = 0 ; i < sizeof(lpFileName) - 1 ; i ++ )
@@ -275,6 +261,11 @@ void *dlopen( const char *file, int mode )
         }
         lpFileName[i] = '\0';
 
+        hCurrentProc = GetCurrentProcess( );
+
+        if( EnumProcessModules( hCurrentProc, NULL, 0, &dwProcModsBefore ) == 0 )
+            dwProcModsBefore = 0;
+
         /* POSIX says the search path is implementation-defined.
          * LOAD_WITH_ALTERED_SEARCH_PATH is used to make it behave more closely
          * to UNIX's search paths (start with system folders instead of current
@@ -283,17 +274,24 @@ void *dlopen( const char *file, int mode )
         hModule = LoadLibraryEx(lpFileName, NULL, 
                                 LOAD_WITH_ALTERED_SEARCH_PATH );
 
-        /* If the object was loaded with RTLD_GLOBAL, add it to list of global
-         * objects, so that its symbols may be retrieved even if the handle for
+        if( EnumProcessModules( hCurrentProc, NULL, 0, &dwProcModsAfter ) == 0 )
+            dwProcModsAfter = 0;
+
+        /* If the object was loaded with RTLD_LOCAL, add it to list of local
+         * objects, so that its symbols cannot be retrieved even if the handle for
          * the original program file is passed. POSIX says that if the same
          * file is specified in multiple invocations, and any of them are
          * RTLD_GLOBAL, even if any further invocations use RTLD_LOCAL, the
-         * symbols will remain global.
+         * symbols will remain global. If number of loaded modules was not
+         * changed after calling LoadLibraryEx(), it means that library was
+         * already loaded.
          */
         if( !hModule )
             save_err_str( lpFileName );
-        else if( (mode & RTLD_GLOBAL) )
-            global_add( &first_object, hModule );
+        else if( (mode & RTLD_LOCAL) && dwProcModsBefore != dwProcModsAfter )
+            local_add( hModule );
+        else if( !(mode & RTLD_LOCAL) && dwProcModsBefore == dwProcModsAfter )
+            local_rem( hModule );
     }
 
     /* Return to previous state of the error-mode bit flags. */
@@ -302,21 +300,6 @@ void *dlopen( const char *file, int mode )
     return (void *) hModule;
 }
 
-static void free_auto( )
-{
-    global_object *pobject = first_automatic_object.next;
-    if( pobject )
-    {
-        global_object *next;
-        for ( ; pobject; pobject = next )
-        {
-            next = pobject->next;
-            free( pobject );
-        }
-        first_automatic_object.next = NULL;
-    }
-}
-
 int dlclose( void *handle )
 {
     HMODULE hModule = (HMODULE) handle;
@@ -326,22 +309,11 @@ int dlclose( void *handle )
 
     ret = FreeLibrary( hModule );
 
-    /* If the object was loaded with RTLD_GLOBAL, remove it from list of global
+    /* If the object was loaded with RTLD_LOCAL, remove it from list of local
      * objects.
      */
     if( ret )
-    {
-        HMODULE cur = GetModuleHandle( NULL );
-        global_rem( &first_object, hModule );
-        if( hModule == cur )
-        {
-            auto_ref_count--;
-            if( auto_ref_count < 0 )
-                auto_ref_count = 0;
-            if( !auto_ref_count )
-                free_auto( );
-        }
-    }
+        local_rem( hModule );
     else
         save_err_ptr_str( handle );
 
@@ -351,10 +323,13 @@ int dlclose( void *handle )
     return (int) ret;
 }
 
+__declspec(noinline) /* Needed for _ReturnAddress() */
 void *dlsym( void *handle, const char *name )
 {
     FARPROC symbol;
+    HMODULE hCaller;
     HMODULE hModule;
+    HANDLE hCurrentProc;
 
 #ifdef UNICODE
     wchar_t namew[MAX_PATH];
@@ -362,39 +337,88 @@ void *dlsym( void *handle, const char *name )
 #endif
 
     current_error = NULL;
+    symbol = NULL;
+    hCaller = NULL;
+    hModule = GetModuleHandle( NULL );
+    hCurrentProc = GetCurrentProcess( );
 
-    symbol = GetProcAddress( (HMODULE) handle, name );
+    if( handle == RTLD_DEFAULT )
+    {
+        /* The symbol lookup happens in the normal global scope; that is,
+         * a search for a symbol using this handle would find the same
+         * definition as a direct use of this symbol in the program code.
+         * So use same lookup procedure as when filename is NULL.
+         */
+        handle = hModule;
+    }
+    else if( handle == RTLD_NEXT )
+    {
+        /* Specifies the next object after this one that defines name.
+         * This one refers to the object containing the invocation of dlsym().
+         * The next object is the one found upon the application of a load
+         * order symbol resolution algorithm. To get caller function of dlsym()
+         * use _ReturnAddress() intrinsic. To get HMODULE of caller function
+         * use undocumented hack from https://stackoverflow.com/a/2396380
+         * The HMODULE of a DLL is the same value as the module's base address.
+         */
+        MEMORY_BASIC_INFORMATION info;
+        size_t sLen;
+        sLen = VirtualQueryEx( hCurrentProc, _ReturnAddress(), &info, sizeof( info ) );
+        if( sLen != sizeof( info ) )
+            goto end;
+        hCaller = (HMODULE) info.AllocationBase;
+        if(!hCaller)
+            goto end;
+    }
 
-    if( symbol != NULL )
-        goto end;
+    if( handle != RTLD_NEXT )
+    {
+        symbol = GetProcAddress( (HMODULE) handle, name );
+
+        if( symbol != NULL )
+            goto end;
+    }
 
     /* If the handle for the original program file is passed, also search
      * in all globally loaded objects.
      */
 
-    hModule = GetModuleHandle( NULL );
-
-    if( hModule == handle )
+    if( hModule == handle || handle == RTLD_NEXT )
     {
-        global_object *pobject;
+        HMODULE *modules;
+        DWORD cbNeeded;
+        DWORD dwSize;
+        size_t i;
 
-        for( pobject = &first_object; pobject; pobject = pobject->next )
+        /* GetModuleHandle( NULL ) only returns the current program file. So
+         * if we want to get ALL loaded module including those in linked DLLs,
+         * we have to use EnumProcessModules( ).
+         */
+        if( EnumProcessModules( hCurrentProc, NULL, 0, &dwSize ) != 0 )
         {
-            if( pobject->hModule )
+            modules = malloc( dwSize );
+            if( modules )
             {
-                symbol = GetProcAddress( pobject->hModule, name );
-                if( symbol != NULL )
-                    goto end;
-            }
-        }
+                if( EnumProcessModules( hCurrentProc, modules, dwSize, &cbNeeded ) != 0 && dwSize == cbNeeded )
+                {
+                    for( i = 0; i < dwSize / sizeof( HMODULE ); i++ )
+                    {
+                        if( handle == RTLD_NEXT && hCaller )
+                        {
+                            /* Next modules can be used for RTLD_NEXT */
+                            if( hCaller == modules[i] )
+                                hCaller = NULL;
+                            continue;
+                        }
+                        if( local_search( modules[i] ) )
+                            continue;
+                        symbol = GetProcAddress( modules[i], name );
+                        if( symbol != NULL )
+                            goto end;
+                    }
 
-        for( pobject = &first_automatic_object; pobject; pobject = pobject->next )
-        {
-            if( pobject->hModule )
-            {
-                symbol = GetProcAddress( pobject->hModule, name );
-                if( symbol != NULL )
-                    goto end;
+                }
+                free( modules );
             }
         }
     }
@@ -471,18 +495,8 @@ char *dlerror( void )
 BOOL WINAPI DllMain( HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved )
 {
     (void) hinstDLL;
-    /*
-     * https://msdn.microsoft.com/en-us/library/windows/desktop/ms682583(v=vs.85).aspx 
-     *
-     *     When handling DLL_PROCESS_DETACH, a DLL should free resources such as heap
-     *     memory only if the DLL is being unloaded dynamically (the lpReserved
-     *     parameter is NULL).
-     */
-    if( fdwReason == DLL_PROCESS_DETACH && !lpvReserved )
-    {
-        auto_ref_count = 0;
-        free_auto( );
-    }
+    (void) fdwReason;
+    (void) lpvReserved;
     return TRUE;
 }
 #endif
